#region usings
using System;
using System.ComponentModel.Composition;
using System.Collections.Generic;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.PluginInterfaces.V2.Graph;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "TagPinExample", Category = "Value", Help = "Basic template with one value in/out", Tags = "c#",AutoEvaluate = true)]
	#endregion PluginInfo
	public class ValueTagPinExampleNode : IPluginEvaluate, IDisposable
	{
		#region fields & pins
		[Input("Input", DefaultValue = 1.0)]
		public ISpread<double> FInput;

		[Output("Output")]
		public ISpread<string> FOutput;

		[Import()]
		public ILogger FLogger;
		
		[Import()]
		public IHDEHost V2Host;
		
		private List<string> output;
		
		private bool Initialized = false;
		#endregion fields & pins
		
		private void OnNodeAdded (INode2 node)
		{
			var pin = node.FindPin("Tag");
			var str = 
				String.Format("Parent Tag: {0} NodePath: {1} Value: {2}", 
					node.Parent.FindPin("Tag")[0],
					node.GetNodePath(false),
					node.FindPin("Tag")[0]);
			output.Add(str);
			FLogger.Log(LogType.Debug,str);
		}

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			if(!Initialized)
			{
				output = new List<string>();
				foreach(var node in V2Host.ExposedNodeService.Nodes)
				{
					OnNodeAdded(node);
				}
				V2Host.ExposedNodeService.NodeAdded += OnNodeAdded;	
				Initialized = true;
			}
			FOutput.SliceCount = output.Count;
			FOutput.AssignFrom(output);
		}
		
		public void Dispose()
		{
			V2Host.ExposedNodeService.NodeAdded -= OnNodeAdded;
		}
	}
}
